from dataclasses import dataclass
from apispec_helper._internal_utils.dataclass_helper_base import DataclassHelperBase
from typing import List
from apispec_helper.basic_type.external_documentation import ExternalDocumentation
from apispec_helper.component.parameter import ParameterType
from apispec_helper.component.request_body import RequestBody
from apispec_helper.component.response import Response
from apispec_helper.basic_type.server import Server

from apispec_helper._internal_utils.one_of_field_checker import OneOfFieldChecker
from apispec_helper._internal_utils.post_init_base import PostInitBase


class OperationsDataclassHelperBase(PostInitBase):
    def __post_init__(self):
        super().__post_init__()
        OneOfFieldChecker(self, ["get", "put", "post", "delete", "options", "head", "patch", "trace"]).execute()


@dataclass
class Operation(DataclassHelperBase):
    # can be "default" or HTTP status code
    responses: dict[str, Response]

    tags: List[str] = None
    summary: str = None
    description: str = None
    externalDocs: ExternalDocumentation = None
    operationId: str = None
    parameters: List[ParameterType] = None
    requestBody: RequestBody = None
    # callback is a special case as callback can be nested, which will be circular dependency. Therefore, set type of
    # callback as dict and let user construct callback object dict for convenience
    # str is for reference
    callbacks: dict | str = None
    deprecated: bool = None
    security: List = None
    servers: List[Server] = None


@dataclass
class Operations(DataclassHelperBase, OperationsDataclassHelperBase):
    get: Operation = None
    put: Operation = None
    post: Operation = None
    delete: Operation = None
    options: Operation = None
    head: Operation = None
    patch: Operation = None
    trace: Operation = None


# to match definition of apispec's core path API
@dataclass
class PathItem(DataclassHelperBase):
    operations: Operations

    summary: str = None
    description: str = None
    servers: List[Server] = None
    parameters: List[ParameterType] = None



from dataclasses import asdict
from apispec_helper.basic_type import CommonType
from apispec_helper.component.parameter import Parameter
from apispec_helper.component.request_body import RequestBody
from typing import Union


ComponentType = Union[CommonType, Parameter, RequestBody]


class ComponentBase:
    def __init__(self, component_definition: ComponentType):
        self.__component_definition = component_definition

    @property
    def component_definition(self):
        return asdict(self.__component_definition)

    @property
    def component_name(self):
        return self.__class__.__name__

    @property
    def apispec_parameter(self) -> dict:
        return {
            "component_id": self.component_name,
            "component": self.component_definition
        }



from apispec_helper._internal_utils.dataclass_helper_base import DataclassHelperBase
from apispec_helper.component.content import ContentType
from apispec_helper.component.header import Header
from apispec_helper.component.link import Link
from dataclasses import dataclass


@dataclass
class Response(DataclassHelperBase):
    description: str
    headers: dict[str, Header] = None
    content: ContentType = None
    links: dict[str, Link] = None



from typing import List


class NoneOfOneOfFieldsProvidedError(Exception):
    def __init__(self, fields: List[str]):
        super().__init__(f"None of one of field provided: [{fields}]")


class OneOfFieldChecker:
    def __init__(self, instance, one_of_fields: List[str]):
        self.__instance = instance
        self.__one_of_fields = one_of_fields

    def execute(self):
        for field in self.__one_of_fields:
            if hasattr(self.__instance, field) is True:
                return

        raise NoneOfOneOfFieldsProvidedError(self.__one_of_fields)



from dataclasses import dataclass
from apispec_helper._internal_utils.dataclass_helper_base import DataclassHelperBase
from apispec_helper.basic_type.common_type import CommonType
from apispec_helper.component.content import ContentType
from apispec_helper.basic_type.examples import ExamplesType
from typing import Union
from apispec_helper._internal_utils.mutual_exclusive_field_checker import MutualExclusiveFieldChecker
from apispec_helper._internal_utils.one_of_field_checker import OneOfFieldChecker
from apispec_helper._internal_utils.post_init_base import PostInitBase


class ParameterDataclassHelperBase(PostInitBase):
    def __post_init__(self):
        super().__post_init__()
        MutualExclusiveFieldChecker(self, ["schema", "content"]).execute()
        OneOfFieldChecker(self, ["schema", "content"]).execute()


class ParameterLocation:
    QUERY = "query"
    PATH = "path"
    COOKIE = "cookie"
    HEADER = "header"


@dataclass
class Parameter(DataclassHelperBase, ParameterDataclassHelperBase):
    # refer to ParameterLocation
    in_: str
    name: str

    schema: CommonType = None
    content: ContentType = None

    description: str = None
    required: bool = None
    deprecated: bool = None
    allowEmptyValue: bool = None

    # refer to ParameterStyle
    style: str = None
    explode: bool = None
    allowReserved: bool = None

    example: dict = None
    examples: ExamplesType = None


# str for reference
ParameterType = Union[Parameter, str]



from dataclasses import dataclass
from apispec_helper._internal_utils.dataclass_helper_base import DataclassHelperBase


@dataclass
class ExternalDocumentation(DataclassHelperBase):
    url: str
    description: str = None



from typing import List


class MultipleMutualExclusiveFieldsProvidedError(Exception):
    def __init__(self, fields: List[str]):
        super().__init__(f"Multiple mutual exclusive field provided: [{fields}]")


class MutualExclusiveFieldChecker:
    def __init__(self, instance, mutual_exclusive_fields: List[str]):
        self.__instance = instance
        self.__mutual_exclusive_fields = mutual_exclusive_fields

    def execute(self):
        mutual_exclusive_check = None

        for field in self.__mutual_exclusive_fields:
            if mutual_exclusive_check is None:
                mutual_exclusive_check = hasattr(self.__instance, field)
            else:
                raise MultipleMutualExclusiveFieldsProvidedError(self.__mutual_exclusive_fields) if mutual_exclusive_check is True else None
